@@ -40,7 +40,7 @@ size_t block_iter_eat_line(BlockIter *bi)
 
 /*
  * Move to beginning of next line.
- * If there is no next line iterator is not advanced.
+ * If there is no next line, iterator is not advanced.
  * Returns number of bytes iterator advanced.
  */
 size_t block_iter_next_line(BlockIter *bi)
@@ -291,10 +291,7 @@ char *block_iter_get_bytes(const BlockIter *bi, size_t len)
 
     while (pos < len) {
         const size_t avail = blk->size - offset;
-        size_t count = len - pos;
-        if (count > avail) {
-            count = avail;
-        }
+        size_t count = MIN(len - pos, avail);
         memcpy(buf + pos, blk->data + offset, count);
         pos += count;
         BUG_ON(pos < len && blk->node.next == bi->head);
@@ -306,25 +303,6 @@ char *block_iter_get_bytes(const BlockIter *bi, size_t len)
 }
 
 // bi should be at bol
-void fill_line_ref(BlockIter *bi, StringView *line)
-{
-    block_iter_normalize(bi);
-    line->data = bi->blk->data + bi->offset;
-    const size_t max = bi->blk->size - bi->offset;
-    if (unlikely(max == 0)) {
-        // Cursor at end of last block
-        line->length = 0;
-        return;
-    }
-    if (bi->blk->nl == 1) {
-        line->length = max - 1;
-        return;
-    }
-    const unsigned char *nl = memchr(line->data, '\n', max);
-    BUG_ON(!nl);
-    line->length = (size_t)(nl - line->data);
-}
-
 void fill_line_nl_ref(BlockIter *bi, StringView *line)
 {
     block_iter_normalize(bi);
@@ -336,18 +314,30 @@ void fill_line_nl_ref(BlockIter *bi, StringView *line)
         return;
     }
     if (bi->blk->nl == 1) {
+        BUG_ON(line->data[max - 1] != '\n');
         line->length = max;
         return;
     }
     const unsigned char *nl = memchr(line->data, '\n', max);
     BUG_ON(!nl);
     line->length = (size_t)(nl - line->data + 1);
+    BUG_ON(line->length == 0);
 }
 
+void fill_line_ref(BlockIter *bi, StringView *line)
+{
+    fill_line_nl_ref(bi, line);
+    // Trim the newline
+    line->length -= (line->length > 0);
+}
+
+// Set the `line` argument to point to the current line and return
+// the offset of the cursor, relative to the start of the line
+// (zero means cursor is at bol)
 size_t fetch_this_line(const BlockIter *bi, StringView *line)
 {
     BlockIter tmp = *bi;
-    const size_t count = block_iter_bol(&tmp);
+    size_t count = block_iter_bol(&tmp);
     fill_line_ref(&tmp, line);
     return count;
 }
