@@ -1,9 +1,11 @@
+#include <stdlib.h>
+#include <string.h>
 #include "change.h"
 #include "buffer.h"
+#include "edit.h"
 #include "error.h"
 #include "util/debug.h"
 #include "util/xmalloc.h"
-#include "view.h"
 
 static ChangeMergeEnum change_merge;
 static ChangeMergeEnum prev_change_merge;
@@ -13,7 +15,7 @@ static Change *alloc_change(void)
     return xcalloc(sizeof(Change));
 }
 
-static void add_change(Change *change)
+static void add_change(Buffer *buffer, Change *change)
 {
     Change *head = buffer->cur_change;
     change->next = head;
@@ -30,7 +32,7 @@ static bool is_change_chain_barrier(const Change *change)
     return !change->ins_count && !change->del_count;
 }
 
-static Change *new_change(void)
+static Change *new_change(Buffer *buffer)
 {
     if (change_barrier) {
         /*
@@ -42,23 +44,23 @@ static Change *new_change(void)
          * chain but then we may have ended up with an empty chain.
          * We don't want to record empty changes ever.
          */
-        add_change(change_barrier);
+        add_change(buffer, change_barrier);
         change_barrier = NULL;
     }
 
     Change *change = alloc_change();
-    add_change(change);
+    add_change(buffer, change);
     return change;
 }
 
-static size_t buffer_offset(void)
+static size_t buffer_offset(const View *view)
 {
     return block_iter_get_offset(&view->cursor);
 }
 
-static void record_insert(size_t len)
+static void record_insert(View *view, size_t len)
 {
-    Change *change = buffer->cur_change;
+    Change *change = view->buffer->cur_change;
     BUG_ON(!len);
     if (
         change_merge == prev_change_merge
@@ -69,17 +71,17 @@ static void record_insert(size_t len)
         return;
     }
 
-    change = new_change();
-    change->offset = buffer_offset();
+    change = new_change(view->buffer);
+    change->offset = buffer_offset(view);
     change->ins_count = len;
 }
 
-static void record_delete(char *buf, size_t len, bool move_after)
+static void record_delete(View *view, char *buf, size_t len, bool move_after)
 {
     BUG_ON(!len);
     BUG_ON(!buf);
 
-    Change *change = buffer->cur_change;
+    Change *change = view->buffer->cur_change;
     if (change_merge == prev_change_merge) {
         if (change_merge == CHANGE_MERGE_DELETE) {
             xrenew(change->buf, change->del_count + len);
@@ -99,21 +101,21 @@ static void record_delete(char *buf, size_t len, bool move_after)
         }
     }
 
-    change = new_change();
-    change->offset = buffer_offset();
+    change = new_change(view->buffer);
+    change->offset = buffer_offset(view);
     change->del_count = len;
     change->move_after = move_after;
     change->buf = buf;
 }
 
-static void record_replace(char *deleted, size_t del_count, size_t ins_count)
+static void record_replace(View *view, char *deleted, size_t del_count, size_t ins_count)
 {
     BUG_ON(del_count && !deleted);
     BUG_ON(!del_count && deleted);
     BUG_ON(!del_count && !ins_count);
 
-    Change *change = new_change();
-    change->offset = buffer_offset();
+    Change *change = new_change(view->buffer);
+    change->offset = buffer_offset(view);
     change->ins_count = ins_count;
     change->del_count = del_count;
     change->buf = deleted;
@@ -139,20 +141,21 @@ void begin_change_chain(void)
     change_merge = CHANGE_MERGE_NONE;
 }
 
-void end_change_chain(void)
+void end_change_chain(View *view)
 {
     if (change_barrier) {
-        // There were no changes in this change chain.
+        // There were no changes in this change chain
         free(change_barrier);
         change_barrier = NULL;
     } else {
-        // There were some changes. Add end of chain marker.
-        add_change(alloc_change());
+        // There were some changes; add end of chain marker
+        add_change(view->buffer, alloc_change());
     }
 }
 
-static void fix_cursors(size_t offset, size_t del, size_t ins)
+static void fix_cursors(const View *view, size_t offset, size_t del, size_t ins)
 {
+    const Buffer *buffer = view->buffer;
     for (size_t i = 0, n = buffer->views.count; i < n; i++) {
         View *v = buffer->views.ptrs[i];
         if (v != view && offset < v->saved_cursor_offset) {
@@ -166,16 +169,16 @@ static void fix_cursors(size_t offset, size_t del, size_t ins)
     }
 }
 
-static void reverse_change(Change *change)
+static void reverse_change(View *view, Change *change)
 {
-    if (buffer->views.count > 1) {
-        fix_cursors(change->offset, change->ins_count, change->del_count);
+    if (view->buffer->views.count > 1) {
+        fix_cursors(view, change->offset, change->ins_count, change->del_count);
     }
 
     block_iter_goto_offset(&view->cursor, change->offset);
     if (!change->ins_count) {
         // Convert delete to insert
-        do_insert(change->buf, change->del_count);
+        do_insert(view, change->buf, change->del_count);
         if (change->move_after) {
             block_iter_skip_bytes(&view->cursor, change->del_count);
         }
@@ -187,22 +190,22 @@ static void reverse_change(Change *change)
         // Reverse replace
         size_t del_count = change->ins_count;
         size_t ins_count = change->del_count;
-        char *buf = do_replace(del_count, change->buf, ins_count);
+        char *buf = do_replace(view, del_count, change->buf, ins_count);
         free(change->buf);
         change->buf = buf;
         change->ins_count = ins_count;
         change->del_count = del_count;
     } else {
         // Convert insert to delete
-        change->buf = do_delete(change->ins_count);
+        change->buf = do_delete(view, change->ins_count, true);
         change->del_count = change->ins_count;
         change->ins_count = 0;
     }
 }
 
-bool undo(void)
+bool undo(View *view)
 {
-    Change *change = buffer->cur_change;
+    Change *change = view->buffer->cur_change;
     view_reset_preferred_x(view);
     if (!change->next) {
         return false;
@@ -215,28 +218,28 @@ bool undo(void)
             if (is_change_chain_barrier(change)) {
                 break;
             }
-            reverse_change(change);
+            reverse_change(view, change);
             count++;
         }
         if (count > 1) {
-            info_msg("Undid %lu changes.", count);
+            info_msg("Undid %lu changes", count);
         }
     } else {
-        reverse_change(change);
+        reverse_change(view, change);
     }
 
-    buffer->cur_change = change->next;
+    view->buffer->cur_change = change->next;
     return true;
 }
 
-bool redo(unsigned long change_id)
+bool redo(View *view, unsigned long change_id)
 {
-    Change *change = buffer->cur_change;
+    Change *change = view->buffer->cur_change;
     view_reset_preferred_x(view);
     if (!change->prev) {
         // Don't complain if change_id is 0
         if (change_id) {
-            error_msg("Nothing to redo.");
+            error_msg("Nothing to redo");
         }
         return false;
     }
@@ -248,12 +251,14 @@ bool redo(unsigned long change_id)
         change_id = nr_prev - 1;
         if (nr_prev > 1) {
             unsigned long i = change_id + 1;
-            info_msg("Redoing newest (%lu) of %lu possible changes.", i, nr_prev);
+            info_msg("Redoing newest (%lu) of %lu possible changes", i, nr_prev);
         }
     } else {
         if (--change_id >= nr_prev) {
-            error_msg("There are only %lu possible changes to redo.", nr_prev);
-            return false;
+            if (nr_prev == 1) {
+                return error_msg("There is only 1 possible change to redo");
+            }
+            return error_msg("There are only %lu possible changes to redo", nr_prev);
         }
     }
 
@@ -265,44 +270,44 @@ bool redo(unsigned long change_id)
             if (is_change_chain_barrier(change)) {
                 break;
             }
-            reverse_change(change);
+            reverse_change(view, change);
             count++;
         }
         if (count > 1) {
-            info_msg("Redid %lu changes.", count);
+            info_msg("Redid %lu changes", count);
         }
     } else {
-        reverse_change(change);
+        reverse_change(view, change);
     }
 
-    buffer->cur_change = change;
+    view->buffer->cur_change = change;
     return true;
 }
 
-void free_changes(Change *ch)
+void free_changes(Change *c)
 {
 top:
-    while (ch->nr_prev) {
-        ch = ch->prev[ch->nr_prev - 1];
+    while (c->nr_prev) {
+        c = c->prev[c->nr_prev - 1];
     }
 
-    // ch is leaf now
-    while (ch->next) {
-        Change *next = ch->next;
-        free(ch->buf);
-        free(ch);
+    // c is leaf now
+    while (c->next) {
+        Change *next = c->next;
+        free(c->buf);
+        free(c);
 
-        ch = next;
-        if (--ch->nr_prev) {
+        c = next;
+        if (--c->nr_prev) {
             goto top;
         }
 
         // We have become leaf
-        free(ch->prev);
+        free(c->prev);
     }
 }
 
-void buffer_insert_bytes(const char *buf, const size_t len)
+void buffer_insert_bytes(View *view, const char *buf, const size_t len)
 {
     view_reset_preferred_x(view);
     if (len == 0) {
@@ -312,23 +317,22 @@ void buffer_insert_bytes(const char *buf, const size_t len)
     size_t rec_len = len;
     if (buf[len - 1] != '\n' && block_iter_is_eof(&view->cursor)) {
         // Force newline at EOF
-        do_insert("\n", 1);
+        do_insert(view, "\n", 1);
         rec_len++;
     }
 
-    do_insert(buf, len);
-    record_insert(rec_len);
+    do_insert(view, buf, len);
+    record_insert(view, rec_len);
 
-    if (buffer->views.count > 1) {
-        fix_cursors(block_iter_get_offset(&view->cursor), len, 0);
+    if (view->buffer->views.count > 1) {
+        fix_cursors(view, block_iter_get_offset(&view->cursor), len, 0);
     }
 }
 
-static bool would_delete_last_bytes(size_t count)
+static bool would_delete_last_bytes(const View *view, size_t count)
 {
     const Block *blk = view->cursor.blk;
     size_t offset = view->cursor.offset;
-
     while (1) {
         size_t avail = blk->size - offset;
         if (avail > count) {
@@ -345,7 +349,7 @@ static bool would_delete_last_bytes(size_t count)
     }
 }
 
-static void buffer_delete_bytes_internal(size_t len, bool move_after)
+static void buffer_delete_bytes_internal(View *view, size_t len, bool move_after)
 {
     view_reset_preferred_x(view);
     if (len == 0) {
@@ -353,7 +357,7 @@ static void buffer_delete_bytes_internal(size_t len, bool move_after)
     }
 
     // Check if all newlines from EOF would be deleted
-    if (would_delete_last_bytes(len)) {
+    if (would_delete_last_bytes(view, len)) {
         BlockIter bi = view->cursor;
         CodePoint u;
         if (block_iter_prev_char(&bi, &u) && u != '\n') {
@@ -364,50 +368,50 @@ static void buffer_delete_bytes_internal(size_t len, bool move_after)
             }
         }
     }
-    record_delete(do_delete(len), len, move_after);
+    record_delete(view, do_delete(view, len, true), len, move_after);
 
-    if (buffer->views.count > 1) {
-        fix_cursors(block_iter_get_offset(&view->cursor), len, 0);
+    if (view->buffer->views.count > 1) {
+        fix_cursors(view, block_iter_get_offset(&view->cursor), len, 0);
     }
 }
 
-void buffer_delete_bytes(size_t len)
+void buffer_delete_bytes(View *view, size_t len)
 {
-    buffer_delete_bytes_internal(len, false);
+    buffer_delete_bytes_internal(view, len, false);
 }
 
-void buffer_erase_bytes(size_t len)
+void buffer_erase_bytes(View *view, size_t len)
 {
-    buffer_delete_bytes_internal(len, true);
+    buffer_delete_bytes_internal(view, len, true);
 }
 
-void buffer_replace_bytes(size_t del_count, const char *ins, size_t ins_count)
+void buffer_replace_bytes(View *view, size_t del_count, const char *ins, size_t ins_count)
 {
     view_reset_preferred_x(view);
     if (del_count == 0) {
-        buffer_insert_bytes(ins, ins_count);
+        buffer_insert_bytes(view, ins, ins_count);
         return;
     }
     if (ins_count == 0) {
-        buffer_delete_bytes(del_count);
+        buffer_delete_bytes(view, del_count);
         return;
     }
 
     // Check if all newlines from EOF would be deleted
-    if (would_delete_last_bytes(del_count)) {
+    if (would_delete_last_bytes(view, del_count)) {
         if (ins[ins_count - 1] != '\n') {
             // Don't replace last newline
             if (--del_count == 0) {
-                buffer_insert_bytes(ins, ins_count);
+                buffer_insert_bytes(view, ins, ins_count);
                 return;
             }
         }
     }
 
-    char *deleted = do_replace(del_count, ins, ins_count);
-    record_replace(deleted, del_count, ins_count);
+    char *deleted = do_replace(view, del_count, ins, ins_count);
+    record_replace(view, deleted, del_count, ins_count);
 
-    if (buffer->views.count > 1) {
-        fix_cursors(block_iter_get_offset(&view->cursor), del_count, ins_count);
+    if (view->buffer->views.count > 1) {
+        fix_cursors(view, block_iter_get_offset(&view->cursor), del_count, ins_count);
     }
 }
