@@ -3,6 +3,8 @@
 
 #include <stdbool.h>
 #include <stddef.h>
+#include "util/macros.h"
+#include "view.h"
 
 typedef enum {
     CHANGE_MERGE_NONE,
@@ -25,13 +27,13 @@ typedef struct Change {
 void begin_change(ChangeMergeEnum m);
 void end_change(void);
 void begin_change_chain(void);
-void end_change_chain(void);
-bool undo(void);
-bool redo(unsigned long change_id);
-void free_changes(Change *head);
-void buffer_insert_bytes(const char *buf, size_t len);
-void buffer_delete_bytes(size_t len);
-void buffer_erase_bytes(size_t len);
-void buffer_replace_bytes(size_t del_count, const char *ins, size_t ins_count);
+void end_change_chain(View *view) NONNULL_ARGS;
+bool undo(View *view) NONNULL_ARGS WARN_UNUSED_RESULT;
+bool redo(View *view, unsigned long change_id) NONNULL_ARGS WARN_UNUSED_RESULT;
+void free_changes(Change *c) NONNULL_ARGS;
+void buffer_insert_bytes(View *view, const char *buf, size_t len) NONNULL_ARG(1);
+void buffer_delete_bytes(View *view, size_t len) NONNULL_ARGS;
+void buffer_erase_bytes(View *view, size_t len) NONNULL_ARGS;
+void buffer_replace_bytes(View *view, size_t del_count, const char *ins, size_t ins_count) NONNULL_ARG(1);
 
 #endif
