@@ -5,7 +5,6 @@
 #include <stddef.h>
 #include "util/macros.h"
 
-NONNULL_ARGS
-bool ec_pattern_match(const char *pattern, size_t pat_len, const char *path);
+bool ec_pattern_match(const char *pattern, size_t pat_len, const char *path) NONNULL_ARGS WARN_UNUSED_RESULT;
 
 #endif
