@@ -9,6 +9,7 @@ static const struct FileExtensionMap {
     {"asm", ASM},
     {"auk", AWK},
     {"automount", INI},
+    {"avsc", JSON},
     {"awk", AWK},
     {"bash", SH},
     {"bat", BATCH},
@@ -17,15 +18,20 @@ static const struct FileExtensionMap {
     {"btm", BATCH},
     {"c++", CPLUSPLUS},
     {"cc", CPLUSPLUS},
+    {"cjs", JAVASCRIPT},
     {"cl", LISP},
     {"clj", CLOJURE},
     {"cls", TEX},
     {"cmake", CMAKE},
     {"cmd", BATCH},
+    {"cnc", GCODE},
+    {"cocci", COCCINELLE},
     {"coffee", COFFEESCRIPT},
     {"cpp", CPLUSPLUS},
     {"cr", RUBY},
     {"cs", CSHARP},
+    {"csh", CSH},
+    {"cshrc", CSH},
     {"cson", COFFEESCRIPT},
     {"css", CSS},
     {"csv", CSV},
@@ -34,6 +40,7 @@ static const struct FileExtensionMap {
     {"desktop", INI},
     {"di", D},
     {"diff", DIFF},
+    {"dircolors", CONFIG},
     {"doap", XML},
     {"docbook", XML},
     {"docker", DOCKER},
@@ -51,15 +58,19 @@ static const struct FileExtensionMap {
     {"erl", ERLANG},
     {"ex", ELIXIR},
     {"exs", ELIXIR},
+    {"fish", FISH},
     {"flatpakref", INI},
     {"frag", GLSL},
     {"gawk", AWK},
+    {"gco", GCODE},
     {"gcode", GCODE},
     {"gemspec", RUBY},
     {"geojson", JSON},
+    {"gitignore", GITIGNORE},
     {"glsl", GLSL},
     {"glslf", GLSL},
     {"glslv", GLSL},
+    {"gltf", JSON},
     {"gnuplot", GNUPLOT},
     {"go", GO},
     {"gp", GNUPLOT},
@@ -68,6 +79,7 @@ static const struct FileExtensionMap {
     {"groovy", GROOVY},
     {"gsed", SED},
     {"gv", DOT},
+    {"har", JSON},
     {"hh", CPLUSPLUS},
     {"hpp", CPLUSPLUS},
     {"hrl", ERLANG},
@@ -80,7 +92,10 @@ static const struct FileExtensionMap {
     {"java", JAVA},
     {"jl", JULIA},
     {"js", JAVASCRIPT},
+    {"jslib", JAVASCRIPT},
     {"json", JSON},
+    {"jsonl", JSON},
+    {"jspre", JAVASCRIPT},
     {"ksh", SH},
     {"kt", KOTLIN},
     {"kts", KOTLIN},
@@ -93,7 +108,9 @@ static const struct FileExtensionMap {
     {"make", MAKE},
     {"markdown", MARKDOWN},
     {"mawk", AWK},
+    {"mcmeta", JSON},
     {"md", MARKDOWN},
+    {"mdown", MARKDOWN},
     {"mk", MAKE},
     {"mkd", MARKDOWN},
     {"mkdn", MARKDOWN},
@@ -102,6 +119,7 @@ static const struct FileExtensionMap {
     {"moon", MOONSCRIPT},
     {"mount", INI},
     {"nawk", AWK},
+    {"nft", NFTABLES},
     {"nginx", NGINX},
     {"nginxconf", NGINX},
     {"nim", NIM},
@@ -140,6 +158,7 @@ static const struct FileExtensionMap {
     {"roff", ROFF},
     {"rs", RUST},
     {"rst", RST},
+    {"sarif", JSON},
     {"scala", SCALA},
     {"scm", SCHEME},
     {"scss", SCSS},
@@ -156,8 +175,11 @@ static const struct FileExtensionMap {
     {"sty", TEX},
     {"supp", CONFIG},
     {"svg", XML},
+    {"swift", SWIFT},
     {"target", INI},
     {"tcl", TCL},
+    {"tcsh", CSH},
+    {"tcshrc", CSH},
     {"tex", TEX},
     {"texi", TEXINFO},
     {"texinfo", TEXINFO},
@@ -193,7 +215,7 @@ static const struct FileExtensionMap {
 
 static FileTypeEnum filetype_from_extension(const StringView ext)
 {
-    if (ext.length >= sizeof(extensions[0].ext)) {
+    if (ext.length == 0 || ext.length >= sizeof(extensions[0].ext)) {
         return NONE;
     }
 
