@@ -7,8 +7,10 @@ typedef enum {
     C, CPLUSPLUS = C,
     CLOJURE,
     CMAKE,
+    COCCINELLE,
     COFFEESCRIPT,
     CONFIG,
+    CSH,
     CSHARP,
     CSS,
     CSV,
@@ -21,13 +23,16 @@ typedef enum {
     DTE,
     ELIXIR,
     ERLANG,
+    FISH,
     GCODE,
     GETTEXT,
     GITCOMMIT,
+    GITIGNORE,
     GITREBASE,
     GLSL,
     GNUPLOT,
     GO,
+    GOMODULE,
     GPERF,
     GRADLE,
     GROOVY,
@@ -49,6 +54,7 @@ typedef enum {
     MARKDOWN,
     MESON,
     MOONSCRIPT,
+    NFTABLES,
     NGINX,
     NIM,
     NINJA,
@@ -77,6 +83,7 @@ typedef enum {
     SED,
     SH,
     SQL,
+    SWIFT,
     TCL,
     TEX,
     TEXINFO,
@@ -98,7 +105,7 @@ typedef enum {
     NR_BUILTIN_FILETYPES
 } FileTypeEnum;
 
-static const char builtin_filetype_names[NR_BUILTIN_FILETYPES][16] = {
+static const char builtin_filetype_names[NR_BUILTIN_FILETYPES][12] = {
     [ADA] = "ada",
     [ASM] = "asm",
     [AWK] = "awk",
@@ -107,8 +114,10 @@ static const char builtin_filetype_names[NR_BUILTIN_FILETYPES][16] = {
     [C] = "c",
     [CLOJURE] = "clojure",
     [CMAKE] = "cmake",
-    [COFFEESCRIPT] = "coffeescript",
+    [COCCINELLE] = "coccinelle",
+    [COFFEESCRIPT] = "coffee",
     [CONFIG] = "config",
+    [CSH] = "csh",
     [CSHARP] = "csharp",
     [CSS] = "css",
     [CSV] = "csv",
@@ -121,13 +130,16 @@ static const char builtin_filetype_names[NR_BUILTIN_FILETYPES][16] = {
     [DTE] = "dte",
     [ELIXIR] = "elixir",
     [ERLANG] = "erlang",
+    [FISH] = "fish",
     [GCODE] = "gcode",
     [GETTEXT] = "gettext",
     [GITCOMMIT] = "gitcommit",
+    [GITIGNORE] = "gitignore",
     [GITREBASE] = "gitrebase",
     [GLSL] = "glsl",
     [GNUPLOT] = "gnuplot",
     [GO] = "go",
+    [GOMODULE] = "gomod",
     [GPERF] = "gperf",
     [GRADLE] = "gradle",
     [GROOVY] = "groovy",
@@ -149,6 +161,7 @@ static const char builtin_filetype_names[NR_BUILTIN_FILETYPES][16] = {
     [MARKDOWN] = "markdown",
     [MESON] = "meson",
     [MOONSCRIPT] = "moonscript",
+    [NFTABLES] = "nftables",
     [NGINX] = "nginx",
     [NIM] = "nim",
     [NINJA] = "ninja",
@@ -177,6 +190,7 @@ static const char builtin_filetype_names[NR_BUILTIN_FILETYPES][16] = {
     [SED] = "sed",
     [SH] = "sh",
     [SQL] = "sql",
+    [SWIFT] = "swift",
     [TCL] = "tcl",
     [TEXINFO] = "texinfo",
     [TEXMFCNF] = "texmfcnf",
