@@ -2,62 +2,80 @@
 #include "editor.h"
 #include "util/unicode.h"
 
-void init_selection(const View *v, SelectionInfo *info)
+static bool include_cursor_char_in_selection(const View *view)
 {
-    BlockIter ei;
-    CodePoint u;
+    const EditorState *e = view->window->editor;
+    if (!e->options.select_cursor_char) {
+        return false;
+    }
 
-    info->so = v->sel_so;
-    info->eo = block_iter_get_offset(&v->cursor);
-    info->si = v->cursor;
+    bool overwrite = view->buffer->options.overwrite;
+    CursorInputMode mode = overwrite ? CURSOR_MODE_OVERWRITE : CURSOR_MODE_INSERT;
+    TermCursorType type = e->cursor_styles[mode].type;
+    if (type == CURSOR_KEEP) {
+        type = e->cursor_styles[CURSOR_MODE_DEFAULT].type;
+    }
+
+    // If "select-cursor-char" option is true, include character under cursor
+    // in selections for any cursor type except bars (where it makes no sense
+    // to do so)
+    return !(type == CURSOR_STEADY_BAR || type == CURSOR_BLINKING_BAR);
+}
+
+void init_selection(const View *view, SelectionInfo *info)
+{
+    info->so = view->sel_so;
+    info->eo = block_iter_get_offset(&view->cursor);
+    info->si = view->cursor;
     block_iter_goto_offset(&info->si, info->so);
     info->swapped = false;
     if (info->so > info->eo) {
         size_t o = info->so;
         info->so = info->eo;
         info->eo = o;
-        info->si = v->cursor;
+        info->si = view->cursor;
         info->swapped = true;
     }
 
-    ei = info->si;
+    BlockIter ei = info->si;
     block_iter_skip_bytes(&ei, info->eo - info->so);
     if (block_iter_is_eof(&ei)) {
         if (info->so == info->eo) {
             return;
         }
+        CodePoint u;
         info->eo -= block_iter_prev_char(&ei, &u);
     }
-    if (v->selection == SELECT_LINES) {
+
+    if (view->selection == SELECT_LINES) {
         info->so -= block_iter_bol(&info->si);
         info->eo += block_iter_eat_line(&ei);
     } else {
-        if (editor.options.select_cursor_char) {
-            // Character under cursor belongs to the selection
+        if (include_cursor_char_in_selection(view)) {
             info->eo += block_iter_next_column(&ei);
         }
     }
 }
 
-size_t prepare_selection(View *v)
+size_t prepare_selection(View *view)
 {
     SelectionInfo info;
-    init_selection(v, &info);
-    v->cursor = info.si;
+    init_selection(view, &info);
+    view->cursor = info.si;
     return info.eo - info.so;
 }
 
-char *view_get_selection(View *v, size_t *size)
+char *view_get_selection(View *view, size_t *size)
 {
-    char *buf = NULL;
-
-    *size = 0;
-    if (v->selection) {
-        BlockIter save = v->cursor;
-        *size = prepare_selection(v);
-        buf = block_iter_get_bytes(&v->cursor, *size);
-        v->cursor = save;
+    if (view->selection == SELECT_NONE) {
+        *size = 0;
+        return NULL;
     }
+
+    BlockIter save = view->cursor;
+    *size = prepare_selection(view);
+    char *buf = block_iter_get_bytes(&view->cursor, *size);
+    view->cursor = save;
     return buf;
 }
 
