@@ -13,9 +13,9 @@ typedef struct {
     bool swapped;
 } SelectionInfo;
 
-void init_selection(const View *v, SelectionInfo *info);
-size_t prepare_selection(View *v);
-char *view_get_selection(View *v, size_t *size);
+void init_selection(const View *view, SelectionInfo *info);
+size_t prepare_selection(View *view);
+char *view_get_selection(View *view, size_t *size);
 size_t get_nr_selected_lines(const SelectionInfo *info);
 size_t get_nr_selected_chars(const SelectionInfo *info);
 
