@@ -2,18 +2,11 @@
 #include <string.h>
 #include "color.h"
 #include "command/serialize.h"
-#include "completion.h"
+#include "util/array.h"
 #include "util/bsearch.h"
 #include "util/debug.h"
-#include "util/hashmap.h"
-#include "util/macros.h"
-#include "util/str-util.h"
 #include "util/xmalloc.h"
 
-TermColor builtin_colors[NR_BC];
-
-static HashMap hl_colors = HASHMAP_INIT;
-
 static const char builtin_color_names[NR_BC][16] = {
     [BC_ACTIVETAB] = "activetab",
     [BC_COMMANDLINE] = "commandline",
@@ -36,55 +29,50 @@ UNITTEST {
     CHECK_BSEARCH_STR_ARRAY(builtin_color_names, strcmp);
 }
 
-static TermColor *find_real_color(const char *name)
+static TermColor *find_real_color(ColorScheme *colors, const char *name)
 {
-    ssize_t idx = BSEARCH_IDX(name, builtin_color_names, (CompareFunction)strcmp);
+    ssize_t idx = BSEARCH_IDX(name, builtin_color_names, vstrcmp);
     if (idx >= 0) {
-        BUG_ON(idx >= ARRAY_COUNT(builtin_color_names));
-        return &builtin_colors[(BuiltinColorEnum)idx];
+        BUG_ON(idx >= ARRAYLEN(builtin_color_names));
+        return &colors->builtin[idx];
     }
-
-    return hashmap_get(&hl_colors, name);
+    return hashmap_get(&colors->other, name);
 }
 
-void set_highlight_color(const char *name, const TermColor *color)
+static const TermColor *find_real_color_const(const ColorScheme *colors, const char *name)
 {
-    TermColor *c = find_real_color(name);
-    if (c) {
-        *c = *color;
-        return;
-    }
-
-    c = xnew(TermColor, 1);
-    *c = *color;
-    hashmap_insert(&hl_colors, xstrdup(name), c);
+    return find_real_color((ColorScheme*)colors, name);
 }
 
-TermColor *find_color(const char *name)
+void set_highlight_color(ColorScheme *colors, const char *name, const TermColor *color)
 {
-    TermColor *c = find_real_color(name);
+    TermColor *existing = find_real_color(colors, name);
+    if (existing) {
+        *existing = *color;
+    } else {
+        hashmap_insert(&colors->other, xstrdup(name), XMEMDUP(color));
+    }
+}
+
+const TermColor *find_color(const ColorScheme *colors, const char *name)
+{
+    const TermColor *c = find_real_color_const(colors, name);
     if (c) {
         return c;
     }
 
     const char *dot = strchr(name, '.');
-    return dot ? find_real_color(dot + 1) : NULL;
+    return dot ? find_real_color_const(colors, dot + 1) : NULL;
 }
 
-void clear_hl_colors(void)
+void clear_hl_colors(ColorScheme *colors)
 {
-    hashmap_clear(&hl_colors, free);
+    hashmap_clear(&colors->other, free);
 }
 
-void collect_hl_colors(const char *prefix)
+void collect_builtin_colors(PointerArray *a, const char *prefix)
 {
-    for (size_t i = 0; i < NR_BC; i++) {
-        const char *name = builtin_color_names[i];
-        if (str_has_prefix(name, prefix)) {
-            add_completion(xstrdup(name));
-        }
-    }
-    collect_hashmap_keys(&hl_colors, prefix);
+    COLLECT_STRINGS(builtin_color_names, a, prefix);
 }
 
 typedef struct {
@@ -99,7 +87,7 @@ static int hlcolor_cmp(const void *ap, const void *bp)
     return strcmp(a->name, b->name);
 }
 
-static void append_color(String *s, const char *name, const TermColor *color)
+void string_append_hl_color(String *s, const char *name, const TermColor *color)
 {
     string_append_literal(s, "hi ");
     string_append_escaped_arg(s, name, true);
@@ -108,18 +96,19 @@ static void append_color(String *s, const char *name, const TermColor *color)
         string_append_literal(s, "-- ");
     }
     string_append_cstring(s, term_color_to_string(color));
-    string_append_byte(s, '\n');
 }
 
-String dump_hl_colors(void)
+String dump_hl_colors(const ColorScheme *colors)
 {
     String buf = string_new(4096);
     string_append_literal(&buf, "# UI colors:\n");
     for (size_t i = 0; i < NR_BC; i++) {
-        append_color(&buf, builtin_color_names[i], &builtin_colors[i]);
+        string_append_hl_color(&buf, builtin_color_names[i], &colors->builtin[i]);
+        string_append_byte(&buf, '\n');
     }
 
-    const size_t count = hl_colors.count;
+    const HashMap *hl_colors = &colors->other;
+    const size_t count = hl_colors->count;
     if (unlikely(count == 0)) {
         return buf;
     }
@@ -127,7 +116,7 @@ String dump_hl_colors(void)
     // Copy the HashMap entries into an array
     HlColor *array = xnew(HlColor, count);
     size_t n = 0;
-    for (HashMapIter it = hashmap_iter(&hl_colors); hashmap_next(&it); ) {
+    for (HashMapIter it = hashmap_iter(hl_colors); hashmap_next(&it); ) {
         const TermColor *c = it.entry->value;
         array[n++] = (HlColor) {
             .name = it.entry->key,
@@ -141,7 +130,8 @@ String dump_hl_colors(void)
 
     string_append_literal(&buf, "\n# Syntax colors:\n");
     for (size_t i = 0; i < count; i++) {
-        append_color(&buf, array[i].name, &array[i].color);
+        string_append_hl_color(&buf, array[i].name, &array[i].color);
+        string_append_byte(&buf, '\n');
     }
 
     free(array);
