@@ -2,8 +2,9 @@
 #include <string.h>
 #include <sys/types.h>
 #include "highlight.h"
-#include "syntax.h"
 #include "block-iter.h"
+#include "syntax/merge.h"
+#include "syntax/syntax.h"
 #include "util/ascii.h"
 #include "util/debug.h"
 #include "util/str-util.h"
@@ -17,12 +18,12 @@ static bool state_is_valid(const State *st)
 static void mark_state_invalid(void **ptrs, size_t idx)
 {
     const State *st = ptrs[idx];
-    ptrs[idx] = (State *)((uintptr_t)st | 1);
+    ptrs[idx] = (State*)((uintptr_t)st | 1);
 }
 
 static bool states_equal(void **ptrs, size_t idx, const State *b)
 {
-    const State *a = (State *)((uintptr_t)ptrs[idx] & ~(uintptr_t)1);
+    const State *a = (State*)((uintptr_t)ptrs[idx] & ~(uintptr_t)1);
     return a == b;
 }
 
@@ -45,6 +46,7 @@ static bool bufis_icase(const ConditionData *u, const char *buf, size_t len)
 static State *handle_heredoc (
     Syntax *syn,
     State *state,
+    const ColorScheme *cs,
     const char *delim,
     size_t len
 ) {
@@ -57,14 +59,14 @@ static State *handle_heredoc (
 
     SyntaxMerge m = {
         .subsyn = state->heredoc.subsyntax,
-        .return_state = state->a.destination,
+        .return_state = state->default_action.destination,
         .delim = delim,
         .delim_len = len
     };
 
     HeredocState *s = xnew(HeredocState, 1);
     *s = (HeredocState) {
-        .state = merge_syntax(syn, &m),
+        .state = merge_syntax(syn, &m, cs),
         .delim = xmemdup(delim, len),
         .len = len,
     };
@@ -73,14 +75,30 @@ static State *handle_heredoc (
     return s->state;
 }
 
+// Set colors in range [start,end] and return number of colors set
+static size_t set_color_range (
+    const TermColor **colors,
+    const Action *action,
+    size_t start,
+    size_t end
+) {
+    BUG_ON(start > end);
+    const TermColor *color = action->emit_color;
+    for (size_t i = start; i < end; i++) {
+        colors[i] = color;
+    }
+    return end - start;
+}
+
 // Line should be terminated with \n unless it's the last line
-static TermColor **highlight_line (
+static const TermColor **highlight_line (
     Syntax *syn,
     State *state,
+    const ColorScheme *cs,
     const StringView *line_sv,
     State **ret
 ) {
-    static TermColor **colors;
+    static const TermColor **colors;
     static size_t alloc;
     const unsigned char *const line = line_sv->data;
     const size_t len = line_sv->length;
@@ -92,182 +110,148 @@ static TermColor **highlight_line (
         xrenew(colors, alloc);
     }
 
-    while (1) {
-        const Condition *cond;
-        const ConditionData *u;
-        const Action *a;
-        unsigned char ch;
     top:
-        if (i == len) {
-            break;
-        }
-        ch = line[i];
-        for (size_t ci = 0, n = state->conds.count; ci < n; ci++) {
-            cond = state->conds.ptrs[ci];
-            u = &cond->u;
-            a = &cond->a;
-            switch (cond->type) {
-            case COND_CHAR_BUFFER:
-                if (!bitset_contains(u->bitset, ch)) {
-                    break;
-                }
-                if (sidx < 0) {
-                    sidx = i;
-                }
-                colors[i++] = a->emit_color;
-                state = a->destination;
-                goto top;
-            case COND_BUFIS:
-                if (sidx >= 0 && bufis(u, line + sidx, i - sidx)) {
-                    for (size_t idx = sidx; idx < i; idx++) {
-                        colors[idx] = a->emit_color;
-                    }
-                    sidx = -1;
-                    state = a->destination;
-                    goto top;
-                }
-                break;
-            case COND_BUFIS_ICASE:
-                if (sidx >= 0 && bufis_icase(u, line + sidx, i - sidx)) {
-                    for (size_t idx = sidx; idx < i; idx++) {
-                        colors[idx] = a->emit_color;
-                    }
-                    sidx = -1;
-                    state = a->destination;
-                    goto top;
-                }
-                break;
-            case COND_CHAR:
-                if (!bitset_contains(u->bitset, ch)) {
-                    break;
-                }
-                colors[i++] = a->emit_color;
-                sidx = -1;
-                state = a->destination;
-                goto top;
-            case COND_CHAR1:
-                if (u->ch != ch) {
-                    break;
-                }
-                colors[i++] = a->emit_color;
-                sidx = -1;
-                state = a->destination;
-                goto top;
-            case COND_INLIST:
-                if (
-                    sidx >= 0
-                    && hashset_get(&u->str_list->strings, line + sidx, i - sidx)
-                ) {
-                    for (size_t idx = sidx; idx < i; idx++) {
-                        colors[idx] = a->emit_color;
-                    }
-                    sidx = -1;
-                    state = a->destination;
-                    goto top;
-                }
-                break;
-            case COND_RECOLOR: {
-                ssize_t idx = i - u->recolor_len;
-                if (idx < 0) {
-                    idx = 0;
-                }
-                while (idx < i) {
-                    colors[idx++] = a->emit_color;
-                }
-                } break;
-            case COND_RECOLOR_BUFFER:
-                if (sidx >= 0) {
-                    while (sidx < i) {
-                        colors[sidx++] = a->emit_color;
-                    }
-                    sidx = -1;
-                }
-                break;
-            case COND_STR: {
-                size_t slen = u->str.len;
-                size_t end = i + slen;
-                if (len < end || !mem_equal(u->str.buf, line + i, slen)) {
-                    break;
-                }
-                while (i < end) {
-                    colors[i++] = a->emit_color;
-                }
-                sidx = -1;
-                state = a->destination;
-                goto top;
-                }
-            case COND_STR_ICASE: {
-                size_t slen = u->str.len;
-                size_t end = i + slen;
-                if (len < end || !mem_equal_icase(u->str.buf, line + i, slen)) {
-                    break;
-                }
-                while (i < end) {
-                    colors[i++] = a->emit_color;
-                }
-                sidx = -1;
-                state = a->destination;
-                goto top;
-                }
-            case COND_STR2:
-                // Optimized COND_STR (length 2, case sensitive)
-                if (
-                    ch == u->str.buf[0]
-                    && len - i > 1
-                    && line[i + 1] == u->str.buf[1]
-                ) {
-                    colors[i++] = a->emit_color;
-                    colors[i++] = a->emit_color;
-                    sidx = -1;
-                    state = a->destination;
-                    goto top;
-                }
-                break;
-            case COND_HEREDOCEND: {
-                const char *str = u->heredocend.data;
-                size_t slen = u->heredocend.length;
-                size_t end = i + slen;
-                if (len >= end && (slen == 0 || mem_equal(str, line + i, slen))) {
-                    while (i < end) {
-                        colors[i++] = a->emit_color;
-                    }
-                    sidx = -1;
-                    state = a->destination;
-                    goto top;
-                }
-                } break;
-            default:
-                BUG("unhandled condition type");
-            }
-        }
+    if (i >= len) {
+        BUG_ON(i > len);
+        *ret = state;
+        return colors;
+    }
 
-        switch (state->type) {
-        case STATE_EAT:
-            colors[i++] = state->a.emit_color;
-            // fallthrough
-        case STATE_NOEAT:
-            sidx = -1;
-            // fallthrough
-        case STATE_NOEAT_BUFFER:
-            a = &state->a;
-            state = a->destination;
-            break;
-        case STATE_HEREDOCBEGIN:
+    for (size_t ci = 0, n = state->conds.count; ci < n; ci++) {
+        const Condition *cond = state->conds.ptrs[ci];
+        const ConditionData *u = &cond->u;
+        const Action *a = &cond->a;
+        switch (cond->type) {
+        case COND_CHAR_BUFFER:
+            if (!bitset_contains(u->bitset, line[i])) {
+                break;
+            }
             if (sidx < 0) {
                 sidx = i;
             }
-            state = handle_heredoc(syn, state, line + sidx, i - sidx);
+            colors[i++] = a->emit_color;
+            state = a->destination;
+            goto top;
+        case COND_BUFIS:
+            if (sidx < 0 || !bufis(u, line + sidx, i - sidx)) {
+                break;
+            }
+            set_color_range(colors, a, sidx, i);
+            sidx = -1;
+            state = a->destination;
+            goto top;
+        case COND_BUFIS_ICASE:
+            if (sidx < 0 || !bufis_icase(u, line + sidx, i - sidx)) {
+                break;
+            }
+            set_color_range(colors, a, sidx, i);
+            sidx = -1;
+            state = a->destination;
+            goto top;
+        case COND_CHAR:
+            if (!bitset_contains(u->bitset, line[i])) {
+                break;
+            }
+            colors[i++] = a->emit_color;
+            sidx = -1;
+            state = a->destination;
+            goto top;
+        case COND_CHAR1:
+            if (u->ch != line[i]) {
+                break;
+            }
+            colors[i++] = a->emit_color;
+            sidx = -1;
+            state = a->destination;
+            goto top;
+        case COND_INLIST:
+            if (sidx < 0 || !hashset_get(&u->str_list->strings, line + sidx, i - sidx)) {
+                break;
+            }
+            set_color_range(colors, a, sidx, i);
+            sidx = -1;
+            state = a->destination;
+            goto top;
+        case COND_RECOLOR: {
+            size_t start = (i >= u->recolor_len) ? i - u->recolor_len : 0;
+            set_color_range(colors, a, start, i);
+            } break;
+        case COND_RECOLOR_BUFFER:
+            if (sidx >= 0) {
+                set_color_range(colors, a, sidx, i);
+                sidx = -1;
+            }
             break;
-        case STATE_INVALID:
+        case COND_STR: {
+            size_t slen = u->str.len;
+            size_t end = i + slen;
+            if (len < end || !mem_equal(u->str.buf, line + i, slen)) {
+                break;
+            }
+            i += set_color_range(colors, a, i, end);
+            sidx = -1;
+            state = a->destination;
+            goto top;
+            }
+        case COND_STR_ICASE: {
+            size_t slen = u->str.len;
+            size_t end = i + slen;
+            if (len < end || !mem_equal_icase(u->str.buf, line + i, slen)) {
+                break;
+            }
+            i += set_color_range(colors, a, i, end);
+            sidx = -1;
+            state = a->destination;
+            goto top;
+            }
+        case COND_STR2:
+            // Optimized COND_STR (length 2, case sensitive)
+            if (len < i + 2 || !mem_equal(u->str.buf, line + i, 2)) {
+                break;
+            }
+            colors[i++] = a->emit_color;
+            colors[i++] = a->emit_color;
+            sidx = -1;
+            state = a->destination;
+            goto top;
+        case COND_HEREDOCEND: {
+            const char *str = u->heredocend.data;
+            size_t slen = u->heredocend.length;
+            size_t end = i + slen;
+            if (len >= end && (slen == 0 || mem_equal(str, line + i, slen))) {
+                i += set_color_range(colors, a, i, end);
+                sidx = -1;
+                state = a->destination;
+                goto top;
+            }
+            } break;
         default:
-            BUG("unhandled default action type");
+            BUG("unhandled condition type");
         }
     }
 
-    if (ret) {
-        *ret = state;
+    switch (state->type) {
+    case STATE_EAT:
+        colors[i++] = state->default_action.emit_color;
+        // Fallthrough
+    case STATE_NOEAT:
+        sidx = -1;
+        // Fallthrough
+    case STATE_NOEAT_BUFFER:
+        state = state->default_action.destination;
+        break;
+    case STATE_HEREDOCBEGIN:
+        if (sidx < 0) {
+            sidx = i;
+        }
+        state = handle_heredoc(syn, state, cs, line + sidx, i - sidx);
+        break;
+    case STATE_INVALID:
+    default:
+        BUG("unhandled default action type");
     }
-    return colors;
+
+    goto top;
 }
 
 static void resize_line_states(PointerArray *s, size_t count)
@@ -294,9 +278,9 @@ static void block_iter_move_down(BlockIter *bi, size_t count)
     }
 }
 
-static ssize_t fill_hole(Buffer *b, BlockIter *bi, ssize_t sidx, ssize_t eidx)
+static ssize_t fill_hole(Buffer *buffer, BlockIter *bi, const ColorScheme *cs, ssize_t sidx, ssize_t eidx)
 {
-    void **ptrs = b->line_start_states.ptrs;
+    void **ptrs = buffer->line_start_states.ptrs;
     ssize_t idx = sidx;
 
     while (idx < eidx) {
@@ -304,7 +288,7 @@ static ssize_t fill_hole(Buffer *b, BlockIter *bi, ssize_t sidx, ssize_t eidx)
         State *st;
         fill_line_nl_ref(bi, &line);
         block_iter_eat_line(bi);
-        highlight_line(b->syn, ptrs[idx++], &line, &st);
+        highlight_line(buffer->syn, ptrs[idx++], cs, &line, &st);
 
         if (ptrs[idx] == st) {
             // Was not invalidated and didn't change
@@ -325,14 +309,14 @@ static ssize_t fill_hole(Buffer *b, BlockIter *bi, ssize_t sidx, ssize_t eidx)
     return idx - sidx;
 }
 
-void hl_fill_start_states(Buffer *b, size_t line_nr)
+void hl_fill_start_states(Buffer *buffer, const ColorScheme *cs, size_t line_nr)
 {
-    BlockIter bi = BLOCK_ITER_INIT(&b->blocks);
-    PointerArray *s = &b->line_start_states;
+    BlockIter bi = block_iter(buffer);
+    PointerArray *s = &buffer->line_start_states;
     ssize_t current_line = 0;
     ssize_t idx = 0;
 
-    if (!b->syn) {
+    if (!buffer->syn) {
         return;
     }
 
@@ -359,7 +343,7 @@ void hl_fill_start_states(Buffer *b, size_t line_nr)
         current_line = idx;
 
         // NOTE: might not fill entire hole, which is ok
-        ssize_t count = fill_hole(b, &bi, idx, last);
+        ssize_t count = fill_hole(buffer, &bi, cs, idx, last);
         idx += count;
         current_line += count;
     }
@@ -370,8 +354,9 @@ void hl_fill_start_states(Buffer *b, size_t line_nr)
         StringView line;
         fill_line_nl_ref(&bi, &line);
         highlight_line (
-            b->syn,
+            buffer->syn,
             states[s->count - 1],
+            cs,
             &line,
             &states[s->count]
         );
@@ -380,21 +365,22 @@ void hl_fill_start_states(Buffer *b, size_t line_nr)
     }
 }
 
-TermColor **hl_line (
-    Buffer *b,
+const TermColor **hl_line (
+    Buffer *buffer,
+    const ColorScheme *cs,
     const StringView *line,
     size_t line_nr,
     bool *next_changed
 ) {
     *next_changed = false;
-    if (!b->syn) {
+    if (!buffer->syn) {
         return NULL;
     }
 
-    PointerArray *s = &b->line_start_states;
+    PointerArray *s = &buffer->line_start_states;
     BUG_ON(line_nr >= s->count);
     State *next;
-    TermColor **colors = highlight_line(b->syn, s->ptrs[line_nr++], line, &next);
+    const TermColor **colors = highlight_line(buffer->syn, s->ptrs[line_nr++], cs, line, &next);
 
     if (line_nr == s->count) {
         resize_line_states(s, s->count + 1);
@@ -418,9 +404,9 @@ TermColor **hl_line (
 }
 
 // Called after text has been inserted to re-highlight changed lines
-void hl_insert(Buffer *b, size_t first, size_t lines)
+void hl_insert(Buffer *buffer, size_t first, size_t lines)
 {
-    PointerArray *s = &b->line_start_states;
+    PointerArray *s = &buffer->line_start_states;
     size_t last = first + lines;
 
     if (first >= s->count) {
@@ -429,8 +415,8 @@ void hl_insert(Buffer *b, size_t first, size_t lines)
     }
 
     if (last + 1 >= s->count) {
-        // Last already highlighted lines changed.
-        // There's nothing to gain, throw them away.
+        // Last already highlighted lines changed; there's nothing to
+        // gain, so throw them away
         s->count = first + 1;
         return;
     }
@@ -451,9 +437,9 @@ void hl_insert(Buffer *b, size_t first, size_t lines)
 }
 
 // Called after text has been deleted to re-highlight changed lines
-void hl_delete(Buffer *b, size_t first, size_t deleted_nl)
+void hl_delete(Buffer *buffer, size_t first, size_t deleted_nl)
 {
-    PointerArray *s = &b->line_start_states;
+    PointerArray *s = &buffer->line_start_states;
     size_t last = first + deleted_nl;
 
     if (s->count == 1) {
@@ -466,14 +452,14 @@ void hl_delete(Buffer *b, size_t first, size_t deleted_nl)
     }
 
     if (last + 1 >= s->count) {
-        // Last already highlighted lines changed.
-        // There's nothing to gain, throw them away.
+        // Last already highlighted lines changed; there's nothing to
+        // gain, so throw them away
         s->count = first + 1;
         return;
     }
 
-    // There are already highlighted lines after changed lines.
-    // Try to save the work.
+    // There are already highlighted lines after changed lines; try to
+    // save the work
 
     // Remove deleted lines (states)
     if (deleted_nl) {
