@@ -1,15 +1,17 @@
 #include <errno.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <sys/types.h>
 #include <unistd.h>
 #include "tag.h"
-#include "completion.h"
+#include "error.h"
+#include "util/debug.h"
+#include "util/log.h"
 #include "util/path.h"
 #include "util/str-util.h"
 #include "util/xmalloc.h"
 #include "util/xreadwrite.h"
 
-static TagFile *current_tag_file;
 static const char *current_filename; // For sorting tags
 
 static int visibility_cmp(const Tag *a, const Tag *b)
@@ -23,13 +25,13 @@ static int visibility_cmp(const Tag *a, const Tag *b)
 
     // Is tag visibility limited to the current file?
     if (a->local) {
-        a_this_file = current_filename && streq(current_filename, a->filename);
+        a_this_file = current_filename && strview_equal_cstring(&a->filename, current_filename);
     }
     if (b->local) {
-        b_this_file = current_filename && streq(current_filename, b->filename);
+        b_this_file = current_filename && strview_equal_cstring(&b->filename, current_filename);
     }
 
-    // Tags local to other file than current are not interesting.
+    // Tags local to other file than current are not interesting
     if (a->local && !a_this_file) {
         // a is not interesting
         if (b->local && !b_this_file) {
@@ -66,7 +68,7 @@ static int kind_cmp(const Tag *a, const Tag *b)
         return 0;
     }
 
-    // Struct member (m) is not very interesting.
+    // Struct member (m) is not very interesting
     if (a->kind == 'm') {
         return 1;
     }
@@ -74,7 +76,7 @@ static int kind_cmp(const Tag *a, const Tag *b)
         return -1;
     }
 
-    // Global variable (v) is not very interesting.
+    // Global variable (v) is not very interesting
     if (a->kind == 'v') {
         return 1;
     }
@@ -88,10 +90,10 @@ static int kind_cmp(const Tag *a, const Tag *b)
 
 static int tag_cmp(const void *ap, const void *bp)
 {
-    const Tag *a = *(const Tag **)ap;
-    const Tag *b = *(const Tag **)bp;
-    int r = visibility_cmp(a, b);
-    return r ? r : kind_cmp(a, b);
+    const Tag *const *a = ap;
+    const Tag *const *b = bp;
+    int r = visibility_cmp(*a, *b);
+    return r ? r : kind_cmp(*a, *b);
 }
 
 // Find "tags" file from directory path and its parent directories
@@ -126,50 +128,62 @@ static bool tag_file_changed (
     return tf->mtime != st->st_mtime || !streq(tf->filename, filename);
 }
 
-static void tag_file_free(TagFile *tf)
+// Note: does not free `tf` itself
+void tag_file_free(TagFile *tf)
 {
     free(tf->filename);
     free(tf->buf);
-    free(tf);
+    *tf = (TagFile){.filename = NULL};
 }
 
-TagFile *load_tag_file(void)
+static bool load_tag_file(TagFile *tf)
 {
     char path[4096];
-    if (!getcwd(path, sizeof(path) - STRLEN("/tags"))) {
-        return NULL;
+    if (unlikely(!getcwd(path, sizeof(path) - STRLEN("/tags")))) {
+        LOG_ERRNO("getcwd");
+        return false;
     }
 
     int fd = open_tag_file(path);
     if (fd < 0) {
-        return NULL;
+        return false;
     }
 
     struct stat st;
-    if (fstat(fd, &st) != 0 || st.st_size <= 0) {
+    if (unlikely(fstat(fd, &st) != 0)) {
+        LOG_ERRNO("fstat");
         xclose(fd);
-        return NULL;
+        return false;
     }
 
-    if (current_tag_file) {
-        if (tag_file_changed(current_tag_file, path, &st)) {
-            tag_file_free(current_tag_file);
-            current_tag_file = NULL;
-        } else {
+    if (unlikely(st.st_size <= 0)) {
+        xclose(fd);
+        return false;
+    }
+
+    if (tf->filename) {
+        if (!tag_file_changed(tf, path, &st)) {
             xclose(fd);
-            return current_tag_file;
+            return true;
         }
+        tag_file_free(tf);
+        BUG_ON(tf->filename);
     }
 
-    char *buf = xmalloc(st.st_size);
-    ssize_t size = xread(fd, buf, st.st_size);
+    char *buf = malloc(st.st_size);
+    if (unlikely(!buf)) {
+        LOG_ERRNO("malloc");
+        xclose(fd);
+        return false;
+    }
+
+    ssize_t size = xread_all(fd, buf, st.st_size);
     xclose(fd);
     if (size < 0) {
         free(buf);
-        return NULL;
+        return false;
     }
 
-    TagFile *tf = xnew(TagFile, 1);
     *tf = (TagFile) {
         .filename = xstrdup(path),
         .buf = buf,
@@ -177,8 +191,7 @@ TagFile *load_tag_file(void)
         .mtime = st.st_mtime,
     };
 
-    current_tag_file = tf;
-    return current_tag_file;
+    return true;
 }
 
 static void free_tags_cb(Tag *t)
@@ -187,13 +200,13 @@ static void free_tags_cb(Tag *t)
     free(t);
 }
 
-void free_tags(PointerArray *tags)
+static void free_tags(PointerArray *tags)
 {
     ptr_array_free_cb(tags, FREE_FUNC(free_tags_cb));
 }
 
 // Both parameters must be absolute and clean
-static const char *path_relative(const char *filename, const StringView dir)
+static const char *path_slice_relative(const char *filename, const StringView dir)
 {
     if (strncmp(filename, dir.data, dir.length) != 0) {
         // Filename doesn't start with dir
@@ -208,15 +221,15 @@ static const char *path_relative(const char *filename, const StringView dir)
     return NULL;
 }
 
-void tag_file_find_tags (
+static void tag_file_find_tags (
     const TagFile *tf,
     const char *filename,
-    const char *name,
+    const StringView *name,
     PointerArray *tags
 ) {
     Tag *t = xnew(Tag, 1);
     size_t pos = 0;
-    while (next_tag(tf, &pos, name, true, t)) {
+    while (next_tag(tf->buf, tf->size, &pos, name, true, t)) {
         ptr_array_append(tags, t);
         t = xnew(Tag, 1);
     }
@@ -226,34 +239,83 @@ void tag_file_find_tags (
         current_filename = NULL;
     } else {
         StringView dir = path_slice_dirname(tf->filename);
-        current_filename = path_relative(filename, dir);
+        current_filename = path_slice_relative(filename, dir);
     }
     ptr_array_sort(tags, tag_cmp);
     current_filename = NULL;
 }
 
-char *tag_file_get_tag_filename(const TagFile *tagfile, const Tag *tag)
+// Note: this moves ownership of tag->pattern to the generated Message
+// and assigns NULL to the old pointer
+void add_message_for_tag(MessageArray *messages, Tag *tag, const StringView *dir)
 {
-    const StringView dir = path_slice_dirname(tagfile->filename);
-    const size_t tag_filename_len = strlen(tag->filename);
+    BUG_ON(dir->length == 0);
+    BUG_ON(dir->data[0] != '/');
 
-    char *filename = xmalloc(dir.length + tag_filename_len + 2);
-    memcpy(filename, dir.data, dir.length);
-    filename[dir.length] = '/';
-    memcpy(filename + dir.length + 1, tag->filename, tag_filename_len + 1);
-    return filename;
+    static const char prefix[] = "Tag ";
+    size_t prefix_len = sizeof(prefix) - 1;
+    size_t msg_len = prefix_len + tag->name.length;
+    Message *m = xmalloc(sizeof(*m) + msg_len + 1);
+
+    memcpy(m->msg, prefix, prefix_len);
+    memcpy(m->msg + prefix_len, tag->name.data, tag->name.length);
+    m->msg[msg_len] = '\0';
+
+    m->loc = xnew0(FileLocation, 1);
+    m->loc->filename = path_join_sv(dir, &tag->filename, false);
+
+    if (tag->pattern) {
+        m->loc->pattern = tag->pattern; // Message takes ownership
+        tag->pattern = NULL;
+    } else {
+        m->loc->line = tag->lineno;
+    }
+
+    add_message(messages, m);
 }
 
-void collect_tags(const TagFile *tf, const char *prefix)
+size_t tag_lookup(TagFile *tf, const StringView *name, const char *filename, MessageArray *messages)
 {
+    clear_messages(messages);
+    if (!load_tag_file(tf)) {
+        error_msg("No tags file");
+        return 0;
+    }
+
+    // Filename helps to find correct tags
+    PointerArray tags = PTR_ARRAY_INIT;
+    tag_file_find_tags(tf, filename, name, &tags);
+
+    size_t ntags = tags.count;
+    if (ntags == 0) {
+        error_msg("Tag '%.*s' not found", (int)name->length, name->data);
+        return 0;
+    }
+
+    StringView tf_dir = path_slice_dirname(tf->filename);
+    for (size_t i = 0; i < ntags; i++) {
+        Tag *tag = tags.ptrs[i];
+        add_message_for_tag(messages, tag, &tf_dir);
+    }
+
+    free_tags(&tags);
+    return ntags;
+}
+
+void collect_tags(TagFile *tf, PointerArray *a, const StringView *prefix)
+{
+    if (!load_tag_file(tf)) {
+        return;
+    }
+
     Tag t;
     size_t pos = 0;
-    char *prev = NULL;
-    while (next_tag(tf, &pos, prefix, false, &t)) {
-        if (!prev || !streq(prev, t.name)) {
-            add_completion(t.name);
+    StringView prev = STRING_VIEW_INIT;
+    while (next_tag(tf->buf, tf->size, &pos, prefix, false, &t)) {
+        BUG_ON(t.name.length == 0);
+        if (prev.length == 0 || !strview_equal(&t.name, &prev)) {
+            ptr_array_append(a, xstrcut(t.name.data, t.name.length));
             prev = t.name;
-            t.name = NULL;
         }
         free_tag(&t);
     }
