@@ -3,8 +3,9 @@
 #include <unistd.h>
 #include "mode.h"
 #include "util/debug.h"
-#include "util/macros.h"
 
+// These are initialized during early startup and then never changed,
+// so they're deemed an "acceptable" use of globals:
 static bool initialized;
 static struct termios cooked, raw, raw_isig;
 
@@ -38,33 +39,31 @@ bool term_mode_init(void)
     return true;
 }
 
-static void xtcsetattr(const struct termios *t)
+static bool xtcsetattr(const struct termios *t)
 {
     if (unlikely(!initialized)) {
-        return;
+        return true;
     }
 
-    int ret;
+    int r;
     do {
-        ret = tcsetattr(STDIN_FILENO, TCSANOW, t);
-    } while (unlikely(ret != 0 && errno == EINTR));
+        r = tcsetattr(STDIN_FILENO, TCSANOW, t);
+    } while (unlikely(r != 0 && errno == EINTR));
 
-    if (unlikely(ret != 0)) {
-        fatal_error("tcsetattr", errno);
-    }
+    return (r == 0);
 }
 
-void term_raw(void)
+bool term_raw(void)
 {
-    xtcsetattr(&raw);
+    return xtcsetattr(&raw);
 }
 
-void term_raw_isig(void)
+bool term_raw_isig(void)
 {
-    xtcsetattr(&raw_isig);
+    return xtcsetattr(&raw_isig);
 }
 
-void term_cooked(void)
+bool term_cooked(void)
 {
-    xtcsetattr(&cooked);
+    return xtcsetattr(&cooked);
 }
