@@ -14,7 +14,7 @@ static const CodepointRange default_ignorable[] = {
     {0x061c, 0x061c},
     {0x115f, 0x1160},
     {0x17b4, 0x17b5},
-    {0x180b, 0x180e},
+    {0x180b, 0x180f},
     {0x200b, 0x200f},
     {0x202a, 0x202e},
     {0x2060, 0x206f},
@@ -53,7 +53,8 @@ static const CodepointRange nonspacing_mark[] = {
     {0x0825, 0x0827},
     {0x0829, 0x082d},
     {0x0859, 0x085b},
-    {0x08d3, 0x08e1},
+    {0x0898, 0x089f},
+    {0x08ca, 0x08e1},
     {0x08e3, 0x0902},
     {0x093a, 0x093a},
     {0x093c, 0x093c},
@@ -94,6 +95,7 @@ static const CodepointRange nonspacing_mark[] = {
     {0x0bcd, 0x0bcd},
     {0x0c00, 0x0c00},
     {0x0c04, 0x0c04},
+    {0x0c3c, 0x0c3c},
     {0x0c3e, 0x0c40},
     {0x0c46, 0x0c48},
     {0x0c4a, 0x0c4d},
@@ -119,7 +121,7 @@ static const CodepointRange nonspacing_mark[] = {
     {0x0e47, 0x0e4e},
     {0x0eb1, 0x0eb1},
     {0x0eb4, 0x0ebc},
-    {0x0ec8, 0x0ecd},
+    {0x0ec8, 0x0ece},
     {0x0f18, 0x0f19},
     {0x0f35, 0x0f35},
     {0x0f37, 0x0f37},
@@ -143,7 +145,7 @@ static const CodepointRange nonspacing_mark[] = {
     {0x109d, 0x109d},
     {0x135d, 0x135f},
     {0x1712, 0x1714},
-    {0x1732, 0x1734},
+    {0x1732, 0x1733},
     {0x1752, 0x1753},
     {0x1772, 0x1773},
     {0x17b4, 0x17b5},
@@ -152,6 +154,7 @@ static const CodepointRange nonspacing_mark[] = {
     {0x17c9, 0x17d3},
     {0x17dd, 0x17dd},
     {0x180b, 0x180d},
+    {0x180f, 0x180f},
     {0x1885, 0x1886},
     {0x18a9, 0x18a9},
     {0x1920, 0x1922},
@@ -167,7 +170,7 @@ static const CodepointRange nonspacing_mark[] = {
     {0x1a65, 0x1a6c},
     {0x1a73, 0x1a7c},
     {0x1a7f, 0x1a7f},
-    {0x1ab0, 0x1ac0},
+    {0x1ab0, 0x1ace},
     {0x1b00, 0x1b03},
     {0x1b34, 0x1b34},
     {0x1b36, 0x1b3a},
@@ -190,8 +193,7 @@ static const CodepointRange nonspacing_mark[] = {
     {0x1ced, 0x1ced},
     {0x1cf4, 0x1cf4},
     {0x1cf8, 0x1cf9},
-    {0x1dc0, 0x1df9},
-    {0x1dfb, 0x1dff},
+    {0x1dc0, 0x1dff},
     {0x20d0, 0x20f0},
     {0x2cef, 0x2cf1},
     {0x2d7f, 0x2d7f},
@@ -247,12 +249,17 @@ static const CodepointRange nonspacing_mark[] = {
     {0x10ae5, 0x10ae6},
     {0x10d24, 0x10d27},
     {0x10eab, 0x10eac},
+    {0x10efd, 0x10eff},
     {0x10f46, 0x10f50},
+    {0x10f82, 0x10f85},
     {0x11001, 0x11001},
     {0x11038, 0x11046},
+    {0x11070, 0x11070},
+    {0x11073, 0x11074},
     {0x1107f, 0x11081},
     {0x110b3, 0x110b6},
     {0x110b9, 0x110ba},
+    {0x110c2, 0x110c2},
     {0x11100, 0x11102},
     {0x11127, 0x1112b},
     {0x1112d, 0x11134},
@@ -265,6 +272,7 @@ static const CodepointRange nonspacing_mark[] = {
     {0x11234, 0x11234},
     {0x11236, 0x11237},
     {0x1123e, 0x1123e},
+    {0x11241, 0x11241},
     {0x112df, 0x112df},
     {0x112e3, 0x112ea},
     {0x11300, 0x11301},
@@ -326,12 +334,20 @@ static const CodepointRange nonspacing_mark[] = {
     {0x11d95, 0x11d95},
     {0x11d97, 0x11d97},
     {0x11ef3, 0x11ef4},
+    {0x11f00, 0x11f01},
+    {0x11f36, 0x11f3a},
+    {0x11f40, 0x11f40},
+    {0x11f42, 0x11f42},
+    {0x13440, 0x13440},
+    {0x13447, 0x13455},
     {0x16af0, 0x16af4},
     {0x16b30, 0x16b36},
     {0x16f4f, 0x16f4f},
     {0x16f8f, 0x16f92},
     {0x16fe4, 0x16fe4},
     {0x1bc9d, 0x1bc9e},
+    {0x1cf00, 0x1cf2d},
+    {0x1cf30, 0x1cf46},
     {0x1d167, 0x1d169},
     {0x1d17b, 0x1d182},
     {0x1d185, 0x1d18b},
@@ -348,8 +364,11 @@ static const CodepointRange nonspacing_mark[] = {
     {0x1e01b, 0x1e021},
     {0x1e023, 0x1e024},
     {0x1e026, 0x1e02a},
+    {0x1e08f, 0x1e08f},
     {0x1e130, 0x1e136},
+    {0x1e2ae, 0x1e2ae},
     {0x1e2ec, 0x1e2ef},
+    {0x1e4ec, 0x1e4ef},
     {0x1e8d0, 0x1e8d6},
     {0x1e944, 0x1e94a},
     {0xe0100, 0xe01ef},
@@ -420,8 +439,13 @@ static const CodepointRange double_width[] = {
     {0x17000, 0x187f7},
     {0x18800, 0x18cd5},
     {0x18d00, 0x18d08},
-    {0x1b000, 0x1b11e},
+    {0x1aff0, 0x1aff3},
+    {0x1aff5, 0x1affb},
+    {0x1affd, 0x1affe},
+    {0x1b000, 0x1b122},
+    {0x1b132, 0x1b132},
     {0x1b150, 0x1b152},
+    {0x1b155, 0x1b155},
     {0x1b164, 0x1b167},
     {0x1b170, 0x1b2fb},
     {0x1f004, 0x1f004},
@@ -455,21 +479,21 @@ static const CodepointRange double_width[] = {
     {0x1f6cc, 0x1f6cc},
     {0x1f6d0, 0x1f6d2},
     {0x1f6d5, 0x1f6d7},
+    {0x1f6dc, 0x1f6df},
     {0x1f6eb, 0x1f6ec},
     {0x1f6f4, 0x1f6fc},
     {0x1f7e0, 0x1f7eb},
+    {0x1f7f0, 0x1f7f0},
     {0x1f90c, 0x1f93a},
     {0x1f93c, 0x1f945},
-    {0x1f947, 0x1f978},
-    {0x1f97a, 0x1f9cb},
-    {0x1f9cd, 0x1f9ff},
-    {0x1fa70, 0x1fa74},
-    {0x1fa78, 0x1fa7a},
-    {0x1fa80, 0x1fa86},
-    {0x1fa90, 0x1faa8},
-    {0x1fab0, 0x1fab6},
-    {0x1fac0, 0x1fac2},
-    {0x1fad0, 0x1fad6},
+    {0x1f947, 0x1f9ff},
+    {0x1fa70, 0x1fa7c},
+    {0x1fa80, 0x1fa88},
+    {0x1fa90, 0x1fabd},
+    {0x1fabf, 0x1fac5},
+    {0x1face, 0x1fadb},
+    {0x1fae0, 0x1fae8},
+    {0x1faf0, 0x1faf8},
     {0x20000, 0x2fffd},
     {0x30000, 0x3fffd},
 };
@@ -488,7 +512,6 @@ static const CodepointRange unprintable[] = {
     {0x05c8, 0x05cf},
     {0x05eb, 0x05ee},
     {0x05f5, 0x05ff},
-    {0x061d, 0x061d},
     {0x070e, 0x070e},
     {0x074b, 0x074c},
     {0x07b2, 0x07bf},
@@ -497,9 +520,9 @@ static const CodepointRange unprintable[] = {
     {0x083f, 0x083f},
     {0x085c, 0x085d},
     {0x085f, 0x085f},
-    {0x086b, 0x089f},
-    {0x08b5, 0x08b5},
-    {0x08c8, 0x08d2},
+    {0x086b, 0x086f},
+    {0x088f, 0x088f},
+    {0x0892, 0x0897},
     {0x0984, 0x0984},
     {0x098d, 0x098e},
     {0x0991, 0x0992},
@@ -577,12 +600,13 @@ static const CodepointRange unprintable[] = {
     {0x0c0d, 0x0c0d},
     {0x0c11, 0x0c11},
     {0x0c29, 0x0c29},
-    {0x0c3a, 0x0c3c},
+    {0x0c3a, 0x0c3b},
     {0x0c45, 0x0c45},
     {0x0c49, 0x0c49},
     {0x0c4e, 0x0c54},
     {0x0c57, 0x0c57},
-    {0x0c5b, 0x0c5f},
+    {0x0c5b, 0x0c5c},
+    {0x0c5e, 0x0c5f},
     {0x0c64, 0x0c65},
     {0x0c70, 0x0c76},
     {0x0c8d, 0x0c8d},
@@ -593,11 +617,11 @@ static const CodepointRange unprintable[] = {
     {0x0cc5, 0x0cc5},
     {0x0cc9, 0x0cc9},
     {0x0cce, 0x0cd4},
-    {0x0cd7, 0x0cdd},
+    {0x0cd7, 0x0cdc},
     {0x0cdf, 0x0cdf},
     {0x0ce4, 0x0ce5},
     {0x0cf0, 0x0cf0},
-    {0x0cf3, 0x0cff},
+    {0x0cf4, 0x0cff},
     {0x0d0d, 0x0d0d},
     {0x0d11, 0x0d11},
     {0x0d45, 0x0d45},
@@ -627,7 +651,7 @@ static const CodepointRange unprintable[] = {
     {0x0ebe, 0x0ebf},
     {0x0ec5, 0x0ec5},
     {0x0ec7, 0x0ec7},
-    {0x0ece, 0x0ecf},
+    {0x0ecf, 0x0ecf},
     {0x0eda, 0x0edb},
     {0x0ee0, 0x0eff},
     {0x0f48, 0x0f48},
@@ -661,8 +685,7 @@ static const CodepointRange unprintable[] = {
     {0x13fe, 0x13ff},
     {0x169d, 0x169f},
     {0x16f9, 0x16ff},
-    {0x170d, 0x170d},
-    {0x1715, 0x171f},
+    {0x1716, 0x171e},
     {0x1737, 0x173f},
     {0x1754, 0x175f},
     {0x176d, 0x176d},
@@ -671,7 +694,6 @@ static const CodepointRange unprintable[] = {
     {0x17de, 0x17df},
     {0x17ea, 0x17ef},
     {0x17fa, 0x17ff},
-    {0x180f, 0x180f},
     {0x181a, 0x181f},
     {0x1879, 0x187f},
     {0x18ab, 0x18af},
@@ -691,9 +713,9 @@ static const CodepointRange unprintable[] = {
     {0x1a8a, 0x1a8f},
     {0x1a9a, 0x1a9f},
     {0x1aae, 0x1aaf},
-    {0x1ac1, 0x1aff},
-    {0x1b4c, 0x1b4f},
-    {0x1b7d, 0x1b7f},
+    {0x1acf, 0x1aff},
+    {0x1b4d, 0x1b4f},
+    {0x1b7f, 0x1b7f},
     {0x1bf4, 0x1bfb},
     {0x1c38, 0x1c3a},
     {0x1c4a, 0x1c4c},
@@ -701,7 +723,6 @@ static const CodepointRange unprintable[] = {
     {0x1cbb, 0x1cbc},
     {0x1cc8, 0x1ccf},
     {0x1cfb, 0x1cff},
-    {0x1dfa, 0x1dfa},
     {0x1f16, 0x1f17},
     {0x1f1e, 0x1f1f},
     {0x1f46, 0x1f47},
@@ -722,15 +743,13 @@ static const CodepointRange unprintable[] = {
     {0x2072, 0x2073},
     {0x208f, 0x208f},
     {0x209d, 0x209f},
-    {0x20c0, 0x20cf},
+    {0x20c1, 0x20cf},
     {0x20f1, 0x20ff},
     {0x218c, 0x218f},
     {0x2427, 0x243f},
     {0x244b, 0x245f},
     {0x2b74, 0x2b75},
     {0x2b96, 0x2b96},
-    {0x2c2f, 0x2c2f},
-    {0x2c5f, 0x2c5f},
     {0x2cf4, 0x2cf8},
     {0x2d26, 0x2d26},
     {0x2d28, 0x2d2c},
@@ -746,7 +765,7 @@ static const CodepointRange unprintable[] = {
     {0x2dcf, 0x2dcf},
     {0x2dd7, 0x2dd7},
     {0x2ddf, 0x2ddf},
-    {0x2e53, 0x2e7f},
+    {0x2e5e, 0x2e7f},
     {0x2e9a, 0x2e9a},
     {0x2ef4, 0x2eff},
     {0x2fd6, 0x2fef},
@@ -758,13 +777,14 @@ static const CodepointRange unprintable[] = {
     {0x318f, 0x318f},
     {0x31e4, 0x31ef},
     {0x321f, 0x321f},
-    {0x9ffd, 0x9fff},
     {0xa48d, 0xa48f},
     {0xa4c7, 0xa4cf},
     {0xa62c, 0xa63f},
     {0xa6f8, 0xa6ff},
-    {0xa7c0, 0xa7c1},
-    {0xa7cb, 0xa7f4},
+    {0xa7cb, 0xa7cf},
+    {0xa7d2, 0xa7d2},
+    {0xa7d4, 0xa7d4},
+    {0xa7da, 0xa7f1},
     {0xa82d, 0xa82f},
     {0xa83a, 0xa83f},
     {0xa878, 0xa87f},
@@ -800,11 +820,10 @@ static const CodepointRange unprintable[] = {
     {0xfb3f, 0xfb3f},
     {0xfb42, 0xfb42},
     {0xfb45, 0xfb45},
-    {0xfbc2, 0xfbd2},
-    {0xfd40, 0xfd4f},
+    {0xfbc3, 0xfbd2},
     {0xfd90, 0xfd91},
-    {0xfdc8, 0xfdef},
-    {0xfdfe, 0xfdff},
+    {0xfdc8, 0xfdce},
+    {0xfdd0, 0xfdef},
     {0xfe1a, 0xfe1f},
     {0xfe53, 0xfe53},
     {0xfe67, 0xfe67},
@@ -848,10 +867,20 @@ static const CodepointRange unprintable[] = {
     {0x104fc, 0x104ff},
     {0x10528, 0x1052f},
     {0x10564, 0x1056e},
-    {0x10570, 0x105ff},
+    {0x1057b, 0x1057b},
+    {0x1058b, 0x1058b},
+    {0x10593, 0x10593},
+    {0x10596, 0x10596},
+    {0x105a2, 0x105a2},
+    {0x105b2, 0x105b2},
+    {0x105ba, 0x105ba},
+    {0x105bd, 0x105ff},
     {0x10737, 0x1073f},
     {0x10756, 0x1075f},
-    {0x10768, 0x107ff},
+    {0x10768, 0x1077f},
+    {0x10786, 0x10786},
+    {0x107b1, 0x107b1},
+    {0x107bb, 0x107ff},
     {0x10806, 0x10807},
     {0x10809, 0x10809},
     {0x10836, 0x10836},
@@ -892,14 +921,15 @@ static const CodepointRange unprintable[] = {
     {0x10e7f, 0x10e7f},
     {0x10eaa, 0x10eaa},
     {0x10eae, 0x10eaf},
-    {0x10eb2, 0x10eff},
+    {0x10eb2, 0x10efc},
     {0x10f28, 0x10f2f},
-    {0x10f5a, 0x10faf},
+    {0x10f5a, 0x10f6f},
+    {0x10f8a, 0x10faf},
     {0x10fcc, 0x10fdf},
     {0x10ff7, 0x10fff},
     {0x1104e, 0x11051},
-    {0x11070, 0x1107e},
-    {0x110c2, 0x110cc},
+    {0x11076, 0x1107e},
+    {0x110c3, 0x110cc},
     {0x110ce, 0x110cf},
     {0x110e9, 0x110ef},
     {0x110fa, 0x110ff},
@@ -909,7 +939,7 @@ static const CodepointRange unprintable[] = {
     {0x111e0, 0x111e0},
     {0x111f5, 0x111ff},
     {0x11212, 0x11212},
-    {0x1123f, 0x1127f},
+    {0x11242, 0x1127f},
     {0x11287, 0x11287},
     {0x11289, 0x11289},
     {0x1128e, 0x1128e},
@@ -941,11 +971,11 @@ static const CodepointRange unprintable[] = {
     {0x11645, 0x1164f},
     {0x1165a, 0x1165f},
     {0x1166d, 0x1167f},
-    {0x116b9, 0x116bf},
+    {0x116ba, 0x116bf},
     {0x116ca, 0x116ff},
     {0x1171b, 0x1171c},
     {0x1172c, 0x1172f},
-    {0x11740, 0x117ff},
+    {0x11747, 0x117ff},
     {0x1183c, 0x1189f},
     {0x118f3, 0x118fe},
     {0x11907, 0x11908},
@@ -960,8 +990,9 @@ static const CodepointRange unprintable[] = {
     {0x119d8, 0x119d9},
     {0x119e5, 0x119ff},
     {0x11a48, 0x11a4f},
-    {0x11aa3, 0x11abf},
-    {0x11af9, 0x11bff},
+    {0x11aa3, 0x11aaf},
+    {0x11af9, 0x11aff},
+    {0x11b0a, 0x11bff},
     {0x11c09, 0x11c09},
     {0x11c37, 0x11c37},
     {0x11c46, 0x11c4f},
@@ -982,20 +1013,24 @@ static const CodepointRange unprintable[] = {
     {0x11d92, 0x11d92},
     {0x11d99, 0x11d9f},
     {0x11daa, 0x11edf},
-    {0x11ef9, 0x11faf},
+    {0x11ef9, 0x11eff},
+    {0x11f11, 0x11f11},
+    {0x11f3b, 0x11f3d},
+    {0x11f5a, 0x11faf},
     {0x11fb1, 0x11fbf},
     {0x11ff2, 0x11ffe},
     {0x1239a, 0x123ff},
     {0x1246f, 0x1246f},
     {0x12475, 0x1247f},
-    {0x12544, 0x12fff},
-    {0x1342f, 0x1342f},
-    {0x13439, 0x143ff},
+    {0x12544, 0x12f8f},
+    {0x12ff3, 0x12fff},
+    {0x13456, 0x143ff},
     {0x14647, 0x167ff},
     {0x16a39, 0x16a3f},
     {0x16a5f, 0x16a5f},
     {0x16a6a, 0x16a6d},
-    {0x16a70, 0x16acf},
+    {0x16abf, 0x16abf},
+    {0x16aca, 0x16acf},
     {0x16aee, 0x16aef},
     {0x16af6, 0x16aff},
     {0x16b46, 0x16b4f},
@@ -1011,20 +1046,29 @@ static const CodepointRange unprintable[] = {
     {0x16ff2, 0x16fff},
     {0x187f8, 0x187ff},
     {0x18cd6, 0x18cff},
-    {0x18d09, 0x1afff},
-    {0x1b11f, 0x1b14f},
-    {0x1b153, 0x1b163},
+    {0x18d09, 0x1afef},
+    {0x1aff4, 0x1aff4},
+    {0x1affc, 0x1affc},
+    {0x1afff, 0x1afff},
+    {0x1b123, 0x1b131},
+    {0x1b133, 0x1b14f},
+    {0x1b153, 0x1b154},
+    {0x1b156, 0x1b163},
     {0x1b168, 0x1b16f},
     {0x1b2fc, 0x1bbff},
     {0x1bc6b, 0x1bc6f},
     {0x1bc7d, 0x1bc7f},
     {0x1bc89, 0x1bc8f},
     {0x1bc9a, 0x1bc9b},
-    {0x1bca4, 0x1cfff},
+    {0x1bca4, 0x1ceff},
+    {0x1cf2e, 0x1cf2f},
+    {0x1cf47, 0x1cf4f},
+    {0x1cfc4, 0x1cfff},
     {0x1d0f6, 0x1d0ff},
     {0x1d127, 0x1d128},
-    {0x1d1e9, 0x1d1ff},
-    {0x1d246, 0x1d2df},
+    {0x1d1eb, 0x1d1ff},
+    {0x1d246, 0x1d2bf},
+    {0x1d2d4, 0x1d2df},
     {0x1d2f4, 0x1d2ff},
     {0x1d357, 0x1d35f},
     {0x1d379, 0x1d3ff},
@@ -1050,18 +1094,28 @@ static const CodepointRange unprintable[] = {
     {0x1d7cc, 0x1d7cd},
     {0x1da8c, 0x1da9a},
     {0x1daa0, 0x1daa0},
-    {0x1dab0, 0x1dfff},
+    {0x1dab0, 0x1deff},
+    {0x1df1f, 0x1df24},
+    {0x1df2b, 0x1dfff},
     {0x1e007, 0x1e007},
     {0x1e019, 0x1e01a},
     {0x1e022, 0x1e022},
     {0x1e025, 0x1e025},
-    {0x1e02b, 0x1e0ff},
+    {0x1e02b, 0x1e02f},
+    {0x1e06e, 0x1e08e},
+    {0x1e090, 0x1e0ff},
     {0x1e12d, 0x1e12f},
     {0x1e13e, 0x1e13f},
     {0x1e14a, 0x1e14d},
-    {0x1e150, 0x1e2bf},
+    {0x1e150, 0x1e28f},
+    {0x1e2af, 0x1e2bf},
     {0x1e2fa, 0x1e2fe},
-    {0x1e300, 0x1e7ff},
+    {0x1e300, 0x1e4cf},
+    {0x1e4fa, 0x1e7df},
+    {0x1e7e7, 0x1e7e7},
+    {0x1e7ec, 0x1e7ec},
+    {0x1e7ef, 0x1e7ef},
+    {0x1e7ff, 0x1e7ff},
     {0x1e8c5, 0x1e8c6},
     {0x1e8d7, 0x1e8ff},
     {0x1e94c, 0x1e94f},
@@ -1115,39 +1169,39 @@ static const CodepointRange unprintable[] = {
     {0x1f249, 0x1f24f},
     {0x1f252, 0x1f25f},
     {0x1f266, 0x1f2ff},
-    {0x1f6d8, 0x1f6df},
+    {0x1f6d8, 0x1f6db},
     {0x1f6ed, 0x1f6ef},
     {0x1f6fd, 0x1f6ff},
-    {0x1f774, 0x1f77f},
-    {0x1f7d9, 0x1f7df},
-    {0x1f7ec, 0x1f7ff},
+    {0x1f777, 0x1f77a},
+    {0x1f7da, 0x1f7df},
+    {0x1f7ec, 0x1f7ef},
+    {0x1f7f1, 0x1f7ff},
     {0x1f80c, 0x1f80f},
     {0x1f848, 0x1f84f},
     {0x1f85a, 0x1f85f},
     {0x1f888, 0x1f88f},
     {0x1f8ae, 0x1f8af},
     {0x1f8b2, 0x1f8ff},
-    {0x1f979, 0x1f979},
-    {0x1f9cc, 0x1f9cc},
     {0x1fa54, 0x1fa5f},
     {0x1fa6e, 0x1fa6f},
-    {0x1fa75, 0x1fa77},
-    {0x1fa7b, 0x1fa7f},
-    {0x1fa87, 0x1fa8f},
-    {0x1faa9, 0x1faaf},
-    {0x1fab7, 0x1fabf},
-    {0x1fac3, 0x1facf},
-    {0x1fad7, 0x1faff},
+    {0x1fa7d, 0x1fa7f},
+    {0x1fa89, 0x1fa8f},
+    {0x1fabe, 0x1fabe},
+    {0x1fac6, 0x1facd},
+    {0x1fadc, 0x1fadf},
+    {0x1fae9, 0x1faef},
+    {0x1faf9, 0x1faff},
     {0x1fb93, 0x1fb93},
     {0x1fbcb, 0x1fbef},
     {0x1fbfa, 0x1ffff},
-    {0x2a6de, 0x2a6ff},
-    {0x2b735, 0x2b73f},
+    {0x2a6e0, 0x2a6ff},
+    {0x2b73a, 0x2b73f},
     {0x2b81e, 0x2b81f},
     {0x2cea2, 0x2ceaf},
     {0x2ebe1, 0x2f7ff},
     {0x2fa1e, 0x2ffff},
-    {0x3134b, 0xe0000},
+    {0x3134b, 0x3134f},
+    {0x323b0, 0xe0000},
     {0xe0002, 0xe001f},
     {0xe0080, 0xe00ff},
     {0xe01f0, 0x10ffff},
