@@ -28,7 +28,7 @@ CodePoint u_str_get_char(const unsigned char *str, size_t *idx);
 CodePoint u_get_char(const unsigned char *buf, size_t size, size_t *idx);
 CodePoint u_get_nonascii(const unsigned char *buf, size_t size, size_t *idx);
 
-void u_set_char_raw(char *str, size_t *idx, CodePoint u);
+size_t u_set_char_raw(char *str, size_t *idx, CodePoint u);
 void u_set_char(char *str, size_t *idx, CodePoint u);
 void u_set_hex(char *str, size_t *idx, CodePoint u);
 
