@@ -1,11 +1,9 @@
 #include <errno.h>
 #include <limits.h>
 #include <stdarg.h>
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include "xmalloc.h"
-#include "checked-arith.h"
 #include "debug.h"
 
 static void *check_alloc(void *alloc)
@@ -66,7 +64,7 @@ static char *xvasprintf(const char *format, va_list ap)
     va_end(ap2);
     if (unlikely(n < 0)) {
         goto error;
-    } else if (unlikely(n >= INT_MAX)) {
+    } else if (unlikely(n == INT_MAX)) {
         errno = EOVERFLOW;
         goto error;
     }
