@@ -6,7 +6,7 @@
 #include <sys/types.h>
 #include "macros.h"
 
-NONNULL_ARGS
+NONNULL_ARGS WARN_UNUSED_RESULT
 static inline int xopen(const char *path, int flags, mode_t mode)
 {
     int fd;
@@ -17,8 +17,10 @@ static inline int xopen(const char *path, int flags, mode_t mode)
     return fd;
 }
 
-ssize_t xread(int fd, void *buf, size_t count) NONNULL_ARGS;
-ssize_t xwrite(int fd, const void *buf, size_t count);
+ssize_t xread(int fd, void *buf, size_t count) NONNULL_ARGS WARN_UNUSED_RESULT;
+ssize_t xwrite(int fd, const void *buf, size_t count) NONNULL_ARGS WARN_UNUSED_RESULT;
+ssize_t xread_all(int fd, void *buf, size_t count) NONNULL_ARGS WARN_UNUSED_RESULT;
+ssize_t xwrite_all(int fd, const void *buf, size_t count) WARN_UNUSED_RESULT;
 int xclose(int fd);
 
 #endif
