@@ -6,9 +6,9 @@
 #include "macros.h"
 
 PRINTF(3) NONNULL_ARGS
-size_t xsnprintf(char *restrict buf, size_t len, const char *restrict fmt, ...);
+size_t xsnprintf(char *restrict s, size_t n, const char *restrict fmt, ...);
 
 VPRINTF(3) NONNULL_ARGS
-size_t xvsnprintf(char *restrict buf, size_t len, const char *restrict fmt, va_list ap);
+size_t xvsnprintf(char *restrict s, size_t n, const char *restrict fmt, va_list v);
 
 #endif
