@@ -1,8 +1,11 @@
 #ifndef UTIL_XSTDIO_H
 #define UTIL_XSTDIO_H
 
+#include <errno.h>
+#include <stdarg.h>
 #include <stdbool.h>
 #include <stdio.h>
+#include "debug.h"
 #include "macros.h"
 #include "xreadwrite.h"
 
@@ -23,7 +26,7 @@ static inline FILE *xfopen(const char *path, const char *mode, int flags, mode_t
         flags |= (plus ? O_RDWR : O_WRONLY) | O_CREAT | O_TRUNC;
         break;
     default:
-        BUG("Invalid fopen() mode string: '%s'", mode);
+        BUG("Unknown fopen() mode string: '%s'", mode);
     }
 
     int fd = xopen(path, flags, mask);
@@ -32,10 +35,19 @@ static inline FILE *xfopen(const char *path, const char *mode, int flags, mode_t
     }
 
     FILE *file = fdopen(fd, mode);
-    if (!file) {
+    if (unlikely(!file)) {
+        int e = errno;
         xclose(fd);
+        errno = e;
     }
     return file;
 }
 
+char *xfgets(char *restrict buf, int bufsize, FILE *restrict stream);
+int xfputs(const char *restrict str, FILE *restrict stream);
+int xfputc(int c, FILE *stream);
+int xvfprintf(FILE *restrict stream, const char *restrict fmt, va_list ap) VPRINTF(2);
+int xfprintf(FILE *restrict stream, const char *restrict fmt, ...) PRINTF(2);
+int xfflush(FILE *stream);
+
 #endif
